#include "SDL.h"
#include "SDL_opengl.h"
#include <iostream>

#include "util.h"
#include "coordinate.h"

#include "config.h"


namespace PRJID {

void error() {
   GLenum errorcode;
   const GLubyte *errorstring;


   if ((errorcode= glGetError()) != GL_NO_ERROR) {
      errorstring = gluErrorString(errorcode);
      std::cerr << "GL_ERROR: " << errorstring << std::endl;
   }

}

void ortho2d(float dx,float dy) {
	if (dx==0) { dx=Config::dx(); dy=Config::dy(); }
	//disable stuff
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);


	// change projection to ortho 
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0.0,dx,dy,0.0);

	// back to mv and push
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
}

void ortho2d_off() {
	//enable stuff
	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH_TEST);


	// pop projection matrix
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();


	// pop mv matrix
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void screenshot(int w,int h,unsigned int t) {
	Uint32 rmask, gmask, bmask, amask;
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif
	SDL_Surface *my;
	my=SDL_CreateRGBSurface( SDL_SWSURFACE, w,h, 32, rmask, gmask, bmask, amask);
	glReadBuffer(GL_FRONT);
	glReadPixels(0, 0, w, h, GL_RGBA, GL_UNSIGNED_BYTE, my->pixels);
	std::ostringstream sstr;
	if (t==0) t=time(NULL);
	sstr << SHORTNAME << "_screenshot_" << t << ".bmp";
	SDL_SaveBMP(my,sstr.str().c_str());
}


void bar(C3 start,C3 size) {
	glBegin(GL_QUADS);
   glVertex2i(start.x,start.y);
   glVertex2i(start.x+size.x,start.y);
   glVertex2i(start.x+size.x,start.y+size.y);
   glVertex2i(start.x,start.y+size.y);
   glEnd();
}

void crosshair() {
	static GLuint dl=0;
	if (dl!=0) { glCallList(dl); return; }
	dl = glGenLists(1);
	glNewList(dl,GL_COMPILE_AND_EXECUTE);


   C3 center;
   center.x=320;
   center.y=240;
   int size=10;
   int d=4;


	glBegin(GL_LINES);
		glVertex2i(center.x,center.y-size-d);
		glVertex2i(center.x,center.y-d);
	glEnd();
	glBegin(GL_LINES);
		glVertex2i(center.x,center.y+size+d);
		glVertex2i(center.x,center.y+d);
	glEnd();
	glBegin(GL_LINES);
		glVertex2i(center.x-size-d,center.y);
		glVertex2i(center.x-d,center.y);
	glEnd();
	glBegin(GL_LINES);
		glVertex2i(center.x+size+d,center.y);
		glVertex2i(center.x+d,center.y);
	glEnd();

	glEndList();
}

void lockmouse(){
	SDL_ShowCursor(SDL_DISABLE);
	SDL_WM_GrabInput(SDL_GRAB_ON);
}
void unlockmouse(){
	SDL_ShowCursor(SDL_ENABLE);
	SDL_WM_GrabInput(SDL_GRAB_OFF);
}
void togglemouselock() {
	if (ismouselocked()) {
		unlockmouse();
	} else {
		lockmouse();
	}
}

bool ismouselocked() {
	if (SDL_ShowCursor(SDL_QUERY)==SDL_DISABLE) { return true; }
	return false;
}
C3 interpolate(C3 start,C3 end,int length,int now) {
	float one;
	float perc;
	one=static_cast <float>(length)/100.0f;

	perc=static_cast <float>(now)/one;

	return interpolate(start,end,static_cast<int>(perc));
}

C3 interpolate(C3 start,C3 end,int percent) {
   C3f diff;

   diff=(end-start).tofloat();
   diff/=100;

   C3 r;
   r=start;
   r+=(diff*static_cast<float>(percent)).toint();
   return r;
}




} //namespace
